import java.util.*;
import java.io.*;


/** Klasse zur Verwaltung der Artikel
  * @author Bernhard Fiser 0109815
  * @version 1.0 */
public class ArtikelVerwaltung extends HashMap implements Serializable
{
	private static ArtikelVerwaltung	hmap = null;

	/** Constructor
	  * legt eine neue Hashmap an, in der die Artikelobjekte
	  * gespeichert werden. Der Constructor ist privat und kann daher nicht
	  * von 'auerhalb' instanziert werden. */
	private ArtikelVerwaltung()
	{
		super();
	}

	/** Pseudo-Constructor zum instanzieren eines Objekts.
	  * Die Instanzierung erfolgt nur, falls noch kein Objekt dieser Klasse
	  * existiert, andernfalls wird dieses Objekt einmal erstellt. */
	public static ArtikelVerwaltung getInstance()
	{
		if (hmap != null)
			return(hmap);

		try
		{
			/* Falls mglich AritkelVerwaltungsobjekt, welches die Artikel
			 * beinhaltet, laden */
		        ObjectInputStream objectInput = new ObjectInputStream(new FileInputStream("/home/itpt/a0109815/public_html/servlets/store/artikel.dat"));
			hmap = (ArtikelVerwaltung) objectInput.readObject();
			objectInput.close();
		}
		catch (Exception e)
		{
			/* Falls nicht mglich, dann eine neue, leere Artikelverwaltung
			 * erstellen */
			hmap = new ArtikelVerwaltung();
		}

		return(hmap);
	}

	/** Methode zum speichern der Artikelveraltung
	  * @param PrintWriter out : Interface zum Webclient output
	  * @return true : falls Speichern erfolgreich
	  * @return false : falls Fehler aufgetreten ist */
	public boolean save()
	{
		if (hmap == null)
			return(false);
		try
		{
			/* Datei ffnen */
		        ObjectOutputStream objectOutput = new ObjectOutputStream(new FileOutputStream("/home/itpt/a0109815/public_html/servlets/store/artikel.dat"));
			/* Object speichern */
		    	objectOutput.writeObject(hmap);
			/* Datei schreiben und schlieen */
		        objectOutput.flush();
        		objectOutput.close();
		}
		/* Im Fehlerfall Exception ausgeben und false zurckgeben */
		catch (Exception e) { return(false); }
		return(true);
	}

	/** ndert einen Artikel, sofern vorhanden,
	  * andernfalls wird der Artikel neu zur Hashmap
	  * hinzugefgt */
	public boolean changeArtikel(Artikel a)
	{
		hmap.remove(a.getID());
		hmap.put(a.getID(), a);
		return(true);
	}

	/** lscht einen Artikel von der Hashmap */
	public boolean deleteArtikel(Artikel a)
	{
		/* Falls bergebener Parameter null ist, false zurckgeben */
		if (a == null)
			return(false);
		/* Falls sich Artikel nicht lschen lsst (weil er gar nicht
		 * existiert), dann false zurckgeben */
		if (hmap.remove(a.getID()) == null)
			return(false);
		/* Andernfalls Lschung erfolgreich -> true zurckgeben */
		return(true);
	}

	/** Artikel anhand der Artikelnummer suchen und zurckliefern
	  * @param int id : Artikelnummer des gesuchten Artikels */
	public Artikel getArtikel(int id)
	{
		/* id in Integerobject umwandeln */
		Integer i = new Integer(id);
		/* Artikelobjekt suchen und zurckgeben */
		return((Artikel) hmap.get(i));
	}

	/** Iterator fr die interne Hashmap erstellen
	  * @return Iterator */
	public Iterator iterator()
	{
		return(hmap.values().iterator());
	}
}
