import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


/** Servlet zur Verwaltung des Warenkorbes.
  *
  * @author Bernhard Fiser 0109815
  * @version 1.0
  */
public class WarenkorbServlet extends HttpServlet
{
	private static ArtikelVerwaltung
			av;

	/** Beim Laden des Servlets wird
	  * die Artikelverwaltung instanziert */
	public void init()
	{
		av = ArtikelVerwaltung.getInstance();
	}

	/** DoPost ruft doGet auf, d.h. zwischen Post und Get
	  * wird in diesem Servlet nicht Unterschieden */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		this.doGet(request, response);
	}

	/** DoGet verarbeitet smtliche Aufrufe des Servlets */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		PrintWriter	out;
		HttpSession	session;
		Warenkorb	korb;
		Artikel		a, b;
		String		bg;
		int		id, c, menge, preis;

		/* Contenttype des Outputs setzen und PrintWriter
		 * auslesen */
		response.setContentType("text/html");
		out = response.getWriter();

		/* Sessionobjekt holen, bzw. erstellen, falls noch nicht vorhanden */
		session = request.getSession(true);

		/* Titel des HTML-Formulars generieren */
		out.println(
			"<html><head><meta name=\"author\" content=\"Bernhard Fiser\">" +
			"<meta name=\"date\" content=\"2002-06-09T13:02:00+02:00\">" +
			"<title>Artikelverwaltung:Warenkorb - Bernhard Fiser 0109815</title></head>" +
			"<body bgcolor=\"#ffffff\">\n");
		out.println(
			"<h1>Willkommen in unserem Online-Shop!</h1>");

		/* Falls neue Session begonnen hat, neuen Warenkorb erstellen */
		if (session.isNew())
		{
			korb = new Warenkorb();
			session.putValue("warenkorb", korb);
			out.println("Ihr Warenkorb ist noch leer, Sie k&ouml;nnen nun aus unserem Angebot ausw&auml;hlen.");
		}
		/* Anderenfalls Warenkorb aus dem Sessionobjekt
		 * auslesen. */
		else
		{
			if ((korb = (Warenkorb) session.getValue("warenkorb")) == null)
			{
				/* Warenkorb neu erstellen, falls kein Warenkorb
				 * im Sessionobjekt (tritt ein, falls Cookie gestetzt,
				 * aber Classen vom Server reloaded wurden */
				korb = new Warenkorb();
				session.putValue("warenkorb", korb);
				out.println("Ihr Warenkorb ist noch leer, Sie k&ouml;nnen nun aus unserem Angebot ausw&auml;hlen.");
			}
		}

		/* Falls 'cgi_buy' dem Servlet bergeben wurde,
		 * Waren bestellen, d.h. Lagerbestand reduzieren */
		if (request.getParameter("cgi_buy") != null)
		{
			if ((preis = BestellServlet.bestellen(korb, av)) > 0)
				out.println("<font color=\"green\">Ihre Waren im Gesamtwert von <b>" + (double) preis / 100 + "</b> wurden bestellt.</font><br>");
			if (korb.size() > 0)
				out.println(
					"<br><font color=\"red\">Einige Waren konnten leider nicht bestellt werden, da die Bestellmenge den Lagerbestand &uuml;berschreitet. (Offenbar hat jemand anderer schneller bestellt ;-) " +
					"Wollen Sie die Bestellmenge automatisch an den Lagerbestand anpassen, dann klicken Sie hier " +
					"<form method=\"post\" action=\"WarenkorbServlet\"><input type=\"submit\" name=\"cgi_corr\" value=\"Anpassen\"></form>.</font><br>");
		}

		/* Falls 'cgi_korr' dem Servlet bergeben wurde
		 * Warenkorbmengen an den Lagerbestand anpassen */
		if (request.getParameter("cgi_corr") != null)
			BestellServlet.anpassen(korb, av);

		/* Artikel im Warenkorb ndern oder speichern */
		if (request.getParameter("cgi_chg") != null)
		{
			/* Parameter fr Waren-ID und Menge parsen */
			try
			{
				id = Integer.parseInt(request.getParameter("cgi_id"));
				menge = Integer.parseInt(request.getParameter("cgi_menge"));
			}
			/* Bei Fehler auf illegale Werte setzen */
			catch (NumberFormatException e)
			{
				id = -1;
				menge = 0;
				out.println("<font color=\"red\">Falsche Daten angegeben, Speicherung nicht m&ouml;glich.<br></font>\n");
			}

			/* Falls Werte nicht illegal sind ... */
			if ((id != -1) && (menge != 0))
			{
				/* ... und Artikel im Lager existiert ... */
				if ((a = av.getArtikel(id)) != null)
				{
					/* ... und Lagermenge grer als Bestellmenge ... */
					if (a.getMenge() >= menge)
					{
						/* ... und Artikel noch nicht im Warenkorb ... */
						if (!korb.containsKey(a.getID()))
						{
							/* Artikel in Warenkorb aufnehmen */
							b = new Artikel(a.getID().intValue(), "", 0, menge, "");
							korb.put(a.getID(), b);
						}
						/* sonst Menge des Artikels im Warenkorb ndern */
						else
						{
							b = (Artikel) korb.get(a.getID());
							b.setMenge(menge);
						}
					}
					/* Falls Bestellmenge grer als Lagermenge,
					 * Fehlermeldung ausgeben */
					else
						out.println("<font color=\"red\">Bestellmenge zu hoch.<br></font>");
				}
				/* Falls bestellter Artikel nicht im Produktportfolie,
				 * Fehlermeldung ausgeben */
				else
					out.println("<font color=\"red\">Artikel nicht vorhanden.<br></font>");
			}
		}

		/* Link fr Produktkatalog generieren */
		out.println("Klicken Sie hier um aus unserem <a href=\"ArtikelListeServlet\">Produktkatalog</a> Waren auszuw&auml;hlen.<br>");

		/* Tabelle der vorhandenen Artikel generieren */
		out.println("<table border=\"1\"><tr><th>ID</th><th>Beschreibung</th><th>Einzelpreis</th><th>URL</th><th>Bestellmenge</th><th>Gesamtpreis</th></tr>");
		/* Iterator der Hashmap erstellen */
		Iterator it = korb.values().iterator();

		c = 0;
		/* ber jedes Objekt iterieren */
		while (it.hasNext())
		{
			/* Hintergrundfarbe der Tabelle festlegen */
			if ((c++ % 2) == 0)
				bg = "#aaeeaa";
			else
				bg = "#eeeeaa";
			/* Artikel auslesen aus der Hashmap */
			a = (Artikel) it.next();
			b = av.getArtikel(a.getID().intValue());
			/* Zeile mit den Artikelattribute ausgeben */
			out.println(
				"<form method=\"post\" action=\"WarenkorbServlet\"><input type=\"hidden\" name=\"cgi_id\" value=\"" + b.getID() + "\">" +
				"<tr bgcolor=\"" + bg + "\">" +
				"<td>" + b.getID() + "</td>" +
				"<td>" + b.getDescription() + "</td>" +
				"<td>" + (double) b.getPreis() / 100 + "</td>" +
				"<td><a href=\"" + b.getURL() + "\">" + b.getURL() + "</a></td>" +
				"<td><input type=\"text\" name=\"cgi_menge\" value=\"" + a.getMenge() + "\" size=\"3\"></td>" +
				"<td>" + (double) (b.getPreis() * a.getMenge()) / 100 + "</td>" +
				"<td><input type=\"submit\" name=\"cgi_chg\" value=\"Menge &auml;ndern\"></td>" +
				"</tr></form>\n");
		}
		/* Tabelle und HTML-Doc beenden */
		out.println("</table>");
		out.println(
			"<form method=\"post\" action=\"WarenkorbServlet\"><br><br>Um Ihre ausgew&auml;hlten Waren zu bestellen dr&uuml;cken Sie hier. " +
			"<input type=\"submit\" name=\"cgi_buy\" value=\"Bestellen\"></form>" +
			"</body></html>\n");
	}
}

